// Copyright 1996, 2013 by Jon Dart.  All Rights Reserved.

// Utility to add ECO codes to PGN game files.

#include "board.h"
#include "legal.h"
#include "movegen.h"
#include "options.h"
#include "movearr.h"
#include "notation.h"
#include "chessio.h"
#include "eco.h"
#include "movearr.h"
#include "globals.h"
#include "util.h"
#include "chessio.h"

extern "C"
{
#include <stddef.h>
#include <string.h>
#include <ctype.h>
};
#include <fstream>
#include <iostream>
#include <ctype.h>
#include <map>
#include <vector>

using namespace std;

static void show_usage()
{
	 cerr << "Usage: ecocoder pgn_file" << endl;
}



int CDECL main(int argc, char **argv)
{
    if (!initGlobals(argv[0], false)) {
        cleanupGlobals();
        exit(-1);
    }
    atexit(cleanupGlobals);
   
   Board board;
   ECO eco;
   int arg = 1;

   if (argc <=1)
   {
         show_usage();
	 return -1;
   }
   else
   {
	ifstream pgn_file( argv[arg], ios::in | ios::binary);
	int c;
        ColorType side;
        string result, white, black;
	if (!pgn_file.good()) {
	  cerr << "could not open file " << argv[arg] << endl;
          exit(-1);
	}
        else
	{
            ArasanVector<ChessIO::Header> hdrs;
            while (!pgn_file.eof())
            {
               long first;
               MoveArray moves;
               board.reset();
               side = White;
               while (pgn_file.good() && (c = pgn_file.get()) != EOF)
               {
                   if (c=='[')
                   {
                      pgn_file.putback(c);
                      break;
                   }
               }
               hdrs.removeAll();
               ChessIO::collect_headers(pgn_file,hdrs,first);
               for (;;)
               {
                   static char buf[2048];
                   static char num[20];
                   ChessIO::Token tok = ChessIO::get_next_token(pgn_file,buf,2048);
                   if (tok == ChessIO::Eof) {
                      break;
		   }
                   else if (tok == ChessIO::Number) {
                      strncpy(num,buf,20);
                   }
                   else if (tok == ChessIO::GameMove) {
                      // parse the move
 		      Move m = Notation::value(board,side,buf);
		      MoveImage(m,cerr);
		      cerr << endl;
                      if (IsNull(m) ||
                        !legalMove(board,StartSquare(m),
                            DestSquare(m))) {
                          // echo to both stdout and stderr
  			  cerr << "Illegal move: " << buf << endl;
                          cout << "Illegal move: " << buf << endl;
                          break;
                      }
                      else {
			  BoardState bs = board.state;
                          string img;
			  // convert to SAN
                          Notation::image(board,m,img);
                          moves.add_move(board,bs,m,img,false);
                          board.doMove(m);
                      }
                      side = OppositeColor(side);
                  }
                  else if (tok == ChessIO::Unknown) {
                      fprintf(stderr,"Unrecognized text: %s\n",buf);
                  }
                  else if (tok == ChessIO::Comment) {
                      // ignored for now
                  }
                  else if (tok == ChessIO::Result) {
                       result = buf;
                       break;
                  }
               }
               // output header
               string ecoC = "";
               int found = ChessIO::get_header(hdrs,"ECO",ecoC);
               if (!found) {
                  string name;
                  eco.classify(moves,ecoC,name);
                  if (ecoC != "") {
 		     ChessIO::Header ecoHeader("ECO",ecoC.c_str());
                     hdrs.append(ecoHeader);
                  }
               }         
               else if (ecoC == "") {
		  string name;
                  eco.classify(moves,ecoC,name);
                  if (ecoC != "") {
		    ChessIO::Header ecoHeader("ECO",ecoC.c_str());
                     for (int i = 0; i < hdrs.length(); i++) {
		        const ChessIO::Header &hdr = hdrs[i];
                        if (hdr.tag() == "ECO") { // replace old header
                           hdrs.set(i,ecoHeader);
                           break;
                        }
                     }
                  }
               }
               if (moves.length()>0) ChessIO::store_pgn(cout,moves,result,hdrs);
            }
            pgn_file.close();
        }
   }

   return 0;
}
